#ifndef __TEXTURELOADER_H__
#define __TEXTURELOADER_H__

#define         JO_TGA_24_BITS                              (24)
#define         JO_TGA_32_BITS                              (32)
#define			JO_TGA_SUPPORTED_FORMAT(M)			        ((M) == JO_TGA_24_BITS || (M) == JO_TGA_32_BITS)

#define			JO_TGA_CONVERT_COLOR(TGA, IDX)		        (((*((unsigned char *)(TGA + IDX))))/8 & 0x1f)

#define         JO_TGA_RGB(R, G, B)		                    (0x8000 | ((B)<<10) | ((G)<<5) | (R))

#define			JO_TGA_24BITS_GET_PIXEL(TGA, X, Y, WIDTH)	JO_TGA_RGB(JO_TGA_CONVERT_COLOR(TGA, (((X) * 3) + 2) + ((Y) * (WIDTH * 3))), \
                                                                JO_TGA_CONVERT_COLOR(TGA, (((X) * 3) + 1) + ((Y) * (WIDTH * 3))), \
                                                                JO_TGA_CONVERT_COLOR(TGA, (((X) * 3)) + ((Y) * (WIDTH * 3))))

#define			JO_TGA_32BITS_GET_PIXEL(TGA, X, Y, WIDTH)	JO_TGA_RGB(JO_TGA_CONVERT_COLOR(TGA, ((4*X) + 2) + ((Y) * (4*WIDTH))), \
                                                                JO_TGA_CONVERT_COLOR(TGA, ((4*X) + 1) + ((Y) * (4*WIDTH))), \
                                                                JO_TGA_CONVERT_COLOR(TGA, ((4*X) + 0) + ((Y) * (4*WIDTH))))
# define JO_COLOR_RGB(R,G,B)	(0x8000 | ((B) << 10) | ((G) << 5) | (R))
# define JO_COLOR_Transparent	(0)

#define JO_ZERO(x) (0)
#define JO_TGA_OK (777777)
#define JO_NULL (NULL)


/*
typedef struct
{
    unsigned int width;
    unsigned int height;
    char    format[4];
    int     internalFormat;
    unsigned char *TextureData;
} texture_data;*/


/**** Bytes alignment can be problematic, and since I'm only trying to read the file format, I just went all-short to save time  ****/
typedef struct
{
   unsigned short  idlength;
   unsigned short  colourmaptype;
   unsigned short  datatypecode;
   short colourmaporigin;
   short colourmaplength;
   unsigned short  colourmapdepth;
   short x_origin;
   short y_origin;
   short width;
   short height;
   unsigned short  bitsperpixel;
   unsigned short  imagedescriptor;
} TGA_HEADER;


class private_class
{
    private:
        static  __jo_force_inline unsigned int          jo_tga_get_pixel(const char * const stream, const int x, const int y, const int width, const int bits);
        static void                                     jo_tga_read_contents(jo_img *img, char * stream, const unsigned int transparent_color, const int bits);
        int		                                        jo_tga_loader_from_stream(jo_img *img, char *stream, const unsigned int transparent_color);
        int	                                            __jo_tga_load(jo_img *img, string sub_dir, string filename, char **stream, int *bits);

};



int ReadTGAHeader(ifstream *ibinfile, TGA_HEADER * header)
{
    ibinfile->read((char*)&header->idlength, sizeof(unsigned char));
    ibinfile->read((char*)&header->colourmaptype, sizeof(unsigned char));
    ibinfile->read((char*)&header->datatypecode, sizeof(unsigned char));
    ibinfile->read((char*)&header->colourmaporigin, sizeof(short));
    ibinfile->read((char*)&header->colourmaplength, sizeof(short));
    ibinfile->read((char*)&header->colourmapdepth, sizeof(unsigned char));
    ibinfile->read((char*)&header->x_origin, sizeof(short));
    ibinfile->read((char*)&header->y_origin, sizeof(short));
    ibinfile->read((char*)&header->width, sizeof(short));
    ibinfile->read((char*)&header->height, sizeof(short));
    ibinfile->read((char*)&header->bitsperpixel, sizeof(unsigned char));
    ibinfile->read((char*)&header->imagedescriptor, sizeof(unsigned char));
    return 1;
}


static  __jo_force_inline jo_color         jo_tga_get_pixel(const char * const stream, const int x, const int y, const int width, unsigned short bits)
{
    switch (bits)
    {
        case JO_TGA_32_BITS:
            if (JO_TGA_CONVERT_COLOR(stream, (4*(x) + 3) + ((y) * 4*(width))) <= 0)
                return (JO_COLOR_Transparent);
            return (JO_TGA_32BITS_GET_PIXEL(stream, x, y, width));
        default:
            return (JO_TGA_24BITS_GET_PIXEL(stream, x, y, width));
    }
}


int ReadTGAData(ifstream * file, jo_img * img, unsigned short bits)
{
    //unsigned short transparent_color = 0;

    for (int i=0; i< img->height*img->width; i++)
    {
        if (bits==32)
        {
            char stream[4] = {0};
            file->read((char*)&stream, (4));
            if (JO_TGA_CONVERT_COLOR(stream, 3) <= 0) img->data[(img->height*img->width)-i-1]=JO_COLOR_Transparent;
            else
                img->data[(img->height*img->width)-i-1] = JO_TGA_32BITS_GET_PIXEL(stream,0,0, 4);
        }
        else if (bits==24)
        {
            char stream[3] = {0};
            file->read((char*)&stream, (3));
            /*if (JO_TGA_CONVERT_COLOR(stream, 0)<=0)
            {   cout << "Transparent color?\n";
                img->data[(img->height*img->height)-i-1] = JO_COLOR_Transparent;}
            else*/
            img->data[(img->height*img->width)-i-1] = JO_TGA_24BITS_GET_PIXEL(stream, 0, 0, 3);
        }
    }

    return 1;
}

int ReadTGAFile (string folder, string filename, unsigned int texture_id)
{

    cout << "\nAttempting to read TGA texture files...\n";
    string name;
    name =  folder + filename + ".TGA"; //Temporary : Will break under several conditions
    TGA_HEADER header;
    ifstream ibinfile(name.c_str(), ios::in | ios::binary);

    if (!ibinfile.is_open()) {cout << "ERROR : COULDN'T LOAD FILE " << filename.c_str() << ".TGA\n"; return -1;}

    name = folder + filename + "TEST.TXT";
    //ofstream oTestFile(name.c_str(), ios::out);
    //oTestFile << "Filename : " << filename.c_str() << "\n";

    ReadTGAHeader(&ibinfile, &header);
    //oTestFile << header.width << "x" << header.height << ", " << header.bitsperpixel << ", " << header.imagedescriptor << "\n";
    cout << "\nTexture " << filename << ", size : " << header.width << "x" << header.height << ", pixel depth : " << header.bitsperpixel << ", RLE = " << header.datatypecode << "\n";

    img[texture_id].width = header.width;
    img[texture_id].height = header.height;


    ReadTGAData(&ibinfile, &img[texture_id], header.bitsperpixel);

    /*int idx=0;
    for (int i=0; i< (header.width); i++)
    {
        for (int ii=0; ii< header.height; ii++)
        {
            oTestFile << img[texture_id].data[idx] << " ";
            idx++;
        }
        oTestFile << "\n";

    }*/

    ibinfile.close();
    //oTestFile.close();
    return JO_TGA_OK;
}




/*
bool		    jo_tga_loader(jo_img *img, const char * const sub_dir, const char * const filename, const jo_color transparent_color)
{
	char                    *stream;
	t_tga_error_code	    code;
	int                     bits;

    stream = JO_NULL;
	if ((code = __jo_tga_load(img, sub_dir, filename, &stream, &bits)) != JO_TGA_OK)
		return (code);
    jo_tga_read_contents(img, stream, transparent_color, bits);
	jo_free(stream);
	return (code);
}

t_tga_error_code	__jo_tga_load(jo_img *img, const char * const sub_dir, const char * const filename, char **stream, int *bits)
{
    if (*stream == JO_NULL)
        *stream = jo_fs_read_file_in_dir(filename, sub_dir, JO_NULL);
    if (*stream == JO_NULL)
        return (JO_TGA_UNSUPPORTED_FORMAT);
    img->width = jo_swap_endian_ushort( *((unsigned short *)(*stream + 12)));
    img->height = jo_swap_endian_ushort( *((unsigned short *)(*stream + 14)));
    *bits = (int)(*stream)[16];
    if (!JO_TGA_SUPPORTED_FORMAT(*bits))
	{
#ifdef JO_DEBUG
        jo_core_error("%s: Unsupported %d bits TGA", filename, *bits);
#endif
        if (filename != JO_NULL)
            jo_free(*stream);
		return (JO_TGA_UNSUPPORTED_FORMAT);
	}
    if (img->data == JO_NULL)
		img->data = (unsigned short *)jo_malloc_with_behaviour(img->height * img->width * sizeof(*img->data), JO_MALLOC_TRY_REUSE_BLOCK);
    if (img->data == JO_NULL)
    {
#ifdef JO_DEBUG
        jo_core_error("%s: Out of memory", filename);
#endif
        if (filename != JO_NULL)
            jo_free(*stream);
		return (JO_TGA_OUT_OF_MEMORY);
    }
	return (JO_TGA_OK);
}
*/
#endif
