#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
//#include <SGL.h> --> Update 2017-08-02 : Ditched SGL by just including what's needed in the Def.h file. Some features might be broken, need more testing. If you encounter problems, comment #include "Def.h" and use SGL.h instead (but make sure you point to the file)
#include "Def.h"
/*********************************************************
* Bounding box generator by XL2. Calculates slopes as well. To be used with a 3D game on the Sega Saturn. Just use a function similar to the jo_square_intersect function, but add
* an extra variable for the Z-axis. If there is a slope, refer to a height collision mask to see if there is a collision of not.
* Make sure that you include SGL (SGL.h) in your project! (UPDATE 2017-08-02 : Removed the need to use SGL. Might cause issues and need more testing)
* Use the display command to load the map, then compile and run everything (output is currently set in a .txt file)
* You will have to manually set the path yourself.
* See below (currently at line 103), you must include your JO_ENGINE 3d mesh .h file
* Make sure that your polygons all have normals as well, since it will be used to determine the direction of the polygon and add depth/slopes. You might have to edit your .h file to remove textures and other things (just comment them out)
* TO DO : Auto-collision mask generation
* TO DO : Code clean up (I commented out several lines for my tests with binary writing and other functions)
* TO DO : Merge more bounding boxes to reduce the CPU load.
* TO DO : Add proper BSP
* TO DO : Write to binary to load the map and its collision data on the Saturn on map change
* TO DO : More testing to find out bugs
***********************************/
using namespace std;

typedef unsigned short jo_color;
typedef struct
{
    int x;
    int y;
    int z;
} vector3;

typedef struct
{
    vector3 MaxValue;
    vector3 MinValue;
    vector3 Rotation;
    int     MaskID;
} _BB_Hitbox;

typedef struct
{
    PDATA       data;
}               jo_3d_mesh;

jo_3d_mesh mesh_array[512] = {0};
_BB_Hitbox  BSP_BB[512] = {0};
#include "OBJ2SGL.h"

typedef struct
{
	unsigned short	width;
	unsigned short	height;
	//jo_color        *data;
	jo_color    data[4096]; //It means maximum texture resolution of 64x64, but even at that resolution the Saturn is struggling
}					jo_img;  //For compatibility with JO ENGINE image format

jo_img img[64];
unsigned int total_mesh = 0;

//bool is_BB_mesh = false;

/*Just to prevent errors being returned without having to modify your 3D mesh .h file.
Just use the display function to initialize all the meshes. This will be used for the bounding boxes*/
void jo_3d_mesh_draw(jo_3d_mesh * mesh)
{
        mesh_array[total_mesh].data = mesh->data;
        total_mesh ++;
}

#define __jo_force_inline inline
/*******************
// EDIT : You don't need the header files anymore, you can export straight from .OBJ files. I will not be supporting header files anymore.
Put the JO ENGINE 3d mesh files you want to read here. It can have several meshes in one file.
The display_mesh command within the JO ENGINE file will be used to create an array of all the meshes in the file.
From that you will be able to write to the binary file and create bounding boxes that will be used for your binary space partitioning.
This allows you to translate/scale/rotate only the vertices within your camera or avatar's vicinity, which helps a lot
for the Saturn CPU. Use the "UseNearClip" command in your Mesh file to further help the Saturn VDP1 by not drawing offscreen quads.
*******************/
/******************************************************/
/*#include "SGL/FPSlevel.h"
#include "SGL/FPSlevel2.h"*/
/******************************************************/

/* Masks are used for non-axis aligned objects. It takes more RAM than calculating slopes on the fly, but it saves CPU usage. For more complex map layout, masks should be avoided */
/*int total_Masks = 0;
int Mask[100][200]={0};*/

#include "TEXTLOADER.h"
#include "POL2BB.h"
#include "MAP2WRAM.h"  //Unsupported ATM, might be revisited to write header files instead, so don't use it now
#include "Map2Bin.h"



int main()
{
    stringstream name;
    string inFolder = "IN/JADE/01/";
    vector3 subdivideFactor;
    //bool subdivide = false;

    total_mesh = 0;
    /*********************
    The "display" function in JO ENGINE files generated using the Map Editor will be used here to make a list of all the meshes in the file.
    NOTE (2017-08-11) : You can load .OBJ files directly, so no need for going with the Jo Engine Map Editor.
    *********************/
    //display_fpslevel_mesh();

    /***************
    WIP, Reads an Obj file, convert it to SGL format and then write it to a custom binary file format.
    ***************/


    subdivideFactor.x = 7;    subdivideFactor.y = 4;    subdivideFactor.z = 2;
    total_mesh=Obj2SGL("IN/JADE/JADEGULLY.obj", 120.0, subdivideFactor, true);
    write_map_to_bin(inFolder, "OUT/BIN/SNC/JADE/JADE1");
    poly_2_BB("OUT/BIN/SNC/JADE/JADE1BB");


    subdivideFactor.x = 5;     subdivideFactor.y = 5;    subdivideFactor.z = 3;
    /*total_mesh=Obj2SGL("IN/CRYSTAL/Crystal2.obj", 100.0, subdivideFactor, true);
    write_map_to_bin("IN/CRYSTAL/TEXTURES/", "OUT/BIN/SNC/CRYSTAL/CRYST2");
    //write_MAP_HEADER("TESTCRYSTAL.txt", 1, total_mesh);
    poly_2_BB("OUT/BIN/SNC/CRYSTAL/CRYS2BB"); */


    inFolder = "IN/FPS1/01/";
   /* subdivideFactor.x = 3;     subdivideFactor.y = 1;    subdivideFactor.z = 4;
    total_mesh=Obj2SGL("IN/FPSlevel2.obj", 100.0, subdivideFactor, false); //File you want to convert (in), and if you want to scale it put the scale factor, else put 1.0 to keep a 1:1 ratio
    write_map_to_bin(inFolder, "OUT/BIN/02/MAPT2");   //write_MAP_HEADER("OUT/TESTTEST.h", 1, total_mesh);
    poly_2_BB("OUT/BIN/02/MAP2BB");*/

   /* total_mesh=Obj2SGL("IN/FPSlevel2_LQ.obj", 100.0);
    write_map_to_bin("OUT/BIN/02/LQM2");
    poly_2_BB("OUT/BIN/02/MAP2BB");*/


   /* total_mesh=Obj2SGL("IN/FPSlevel.obj", 100.0, subdivideFactor, false);
   // write_MAP_HEADER("TESTFPS1.txt", 1, total_mesh); //File you want to convert (in), and if you want to scale it put the scale factor, else put 1.0 to keep a 1:1 ratio
    write_map_to_bin(inFolder, "OUT/BIN/01/MAPT");
    poly_2_BB("OUT/BIN/01/MAP1BB");*/

    return 777;
}
